/*
 * Copyright (C) 2009 eCo Software
 * Copyright (C) 2009 Dmitry A.Steklenev
 */

#define  INCL_WIN
#include <os2.h>
#include <stdlib.h>
#include <stdarg.h>
#include <stdio.h>
#include <foc.h>

#include "dirtree.h"

/* Appends specified text to the tracer list box.
 */

static VOID
trace( HWND hwnd, const char* fmt, ... )
{
  CHAR    message[_MAX_PATH+128];
  HWND    tracer = WinWindowFromID( hwnd, LB_TRACER );
  SHORT   pos;
  va_list args;

  va_start( args, fmt );
  vsprintf( message, fmt, args );
  va_end( args );

  if((SHORT)WinSendMsg( tracer, LM_QUERYITEMCOUNT, 0, 0 ) == 32767 ) {
    WinSendMsg( tracer, LM_DELETEITEM, 0, 0 );
  }

  pos = (SHORT)WinSendMsg( tracer, LM_INSERTITEM, MPFROMSHORT( LIT_END ), MPFROMP( message ));
  WinSendMsg( tracer, LM_SELECTITEM,  MPFROMSHORT( pos ), MPFROMLONG( TRUE ));
}

/* Formats dialog presentation window.
 */

static void
format_dlg( HWND hwnd, ULONG msg, MPARAM mp1, MPARAM mp2 )
{
  static HWND  dirtree       = NULLHANDLE;
  static SHORT dirtree_right = 0;
  static SHORT dirtree_top   = 0;
  static HWND  path          = NULLHANDLE;
  static SHORT path_right    = 0;
  static HWND  tracer        = NULLHANDLE;
  static SHORT tracer_right  = 0;

  switch( msg ) {
    case WM_INITDLG:
    {
      SWP swp_dialog;
      SWP swp;

      WinLoadMenu( hwnd, NULLHANDLE, MN_SAMPLE );
      WinSendMsg ( hwnd, WM_UPDATEFRAME, MPFROMLONG( FCF_MENU ), 0 );

      dirtree = WinWindowFromID( hwnd, DT_DIRTREE );
      path    = WinWindowFromID( hwnd, EF_PATH    );
      tracer  = WinWindowFromID( hwnd, LB_TRACER  );

      WinQueryWindowPos( hwnd, &swp_dialog );
      WinQueryWindowPos( dirtree, &swp );
      dirtree_right = swp_dialog.cx - swp.cx;
      dirtree_top = swp_dialog.cy - swp.cy;
      WinQueryWindowPos( path, &swp );
      path_right = swp_dialog.cx - swp.cx;
      WinQueryWindowPos( tracer, &swp );
      tracer_right = swp_dialog.cx - swp.cx;
      break;
    }

    case WM_WINDOWPOSCHANGED:
    {
      PSWP pswp_dialog = (PSWP)mp1;
      if( pswp_dialog->fl & SWP_SIZE )
      {
        SWP swp;

        WinQueryWindowPos( dirtree, &swp );
        swp.cx = pswp_dialog->cx - dirtree_right;
        swp.cy = pswp_dialog->cy - dirtree_top;
        WinSetWindowPos( dirtree, 0, 0, 0, swp.cx, swp.cy, SWP_SIZE );

        WinQueryWindowPos( path, &swp );
        swp.cx = pswp_dialog->cx - path_right - 6;
        WinSetWindowPos( path, 0, 0, 0, swp.cx, swp.cy - 6, SWP_SIZE );

        WinQueryWindowPos( tracer, &swp );
        swp.cx = pswp_dialog->cx - tracer_right;
        WinSetWindowPos( tracer, 0, 0, 0, swp.cx, swp.cy, SWP_SIZE );
      }
      break;
    }
  }
}

/* Processes the dialog messages.
 */

static MRESULT EXPENTRY
process_dlg( HWND hwnd, ULONG msg, MPARAM mp1, MPARAM mp2 )
{
  switch( msg ) {
    case WM_INITDLG:
    case WM_WINDOWPOSCHANGED:
      format_dlg( hwnd, msg, mp1, mp2 );
      break;

    case WM_COMMAND:
      switch( SHORT1FROMMP( mp1 )) {
        case ID_QUERY_CURRENT:
        {
          CHAR szDirName[CCHMAXPATH];
          if( LONGFROMMR( WinSendDlgItemMsg( hwnd, DT_DIRTREE,
                          DTM_QUERYCURDIR, MPFROMLONG( CCHMAXPATH ),  MPFROMP( szDirName ))) > 0 )
          {
            WinSetDlgItemText( hwnd, EF_PATH, szDirName );
          }
          break;
        }

        case ID_REFRESHALL:
          WinSendDlgItemMsg( hwnd, DT_DIRTREE, DTM_REFRESH, MPFROMP( "" ), 0 );
          break;

        case ID_DELETECUR:
        {
          CHAR szDirName[CCHMAXPATH];
          if( LONGFROMMR( WinSendDlgItemMsg( hwnd, DT_DIRTREE,
                          DTM_QUERYCURDIR, MPFROMLONG( CCHMAXPATH ),  MPFROMP( szDirName ))) > 0 )
          {
            WinSendDlgItemMsg( hwnd, DT_DIRTREE, DTM_REQUESTDELETE, MPFROMP( szDirName ), 0 );
          }
          break;
        }

        case ID_RENAMECUR:
        {
          CHAR szDirName[CCHMAXPATH];
          if( LONGFROMMR( WinSendDlgItemMsg( hwnd, DT_DIRTREE,
                          DTM_QUERYCURDIR, MPFROMLONG( CCHMAXPATH ),  MPFROMP( szDirName ))) > 0 )
          {
            WinSendDlgItemMsg( hwnd, DT_DIRTREE, DTM_REQUESTEDIT, MPFROMP( szDirName ), 0 );
          }
          break;
        }

        case ID_CREATENEW:
        {
          CHAR szDirName[CCHMAXPATH];
          if( LONGFROMMR( WinSendDlgItemMsg( hwnd, DT_DIRTREE,
                          DTM_QUERYCURDIR, MPFROMLONG( CCHMAXPATH ),  MPFROMP( szDirName ))) > 0 )
          {
            WinSendDlgItemMsg( hwnd, DT_DIRTREE, DTM_REQUESTNEWDIR, MPFROMP( szDirName ), 0 );
          }
          break;
        }
      }
      return 0;

    case WM_CHAR:
      if(( SHORT1FROMMP(mp1) & KC_VIRTUALKEY ) && !( SHORT1FROMMP(mp1) & ( KC_KEYUP | KC_PREVDOWN ))) {
        if( SHORT2FROMMP(mp2) == VK_ENTER || SHORT2FROMMP(mp2) == VK_NEWLINE )
        {
          CHAR pathname[CCHMAXPATH];
          WinQueryDlgItemText( hwnd, EF_PATH, CCHMAXPATH, pathname );
          WinSendDlgItemMsg( hwnd, DT_DIRTREE, DTM_CHDIR, MPFROMP( pathname ), 0 );
        }
      }
      break;

    case WM_CONTROL:
      if( SHORT1FROMMP(mp1) == DT_DIRTREE ) {
        switch( SHORT2FROMMP(mp1)) {
          case DTN_CHDIR:
            trace( hwnd, "WM_CONTROL [DTN_CHDIR  ] - %s", mp2 );
            break;
          case DTN_DELETED:
            trace( hwnd, "WM_CONTROL [DTN_DELETED] - %s", mp2 );
            break;
          case DTN_NEW:
            trace( hwnd, "WM_CONTROL [DTN_NEW    ] - %s", mp2 );
            break;
          case DTN_RENAMED:
            trace( hwnd, "WM_CONTROL [DTN_RENAMED] - %s -> %s",
                                                          ((PDIRRENAMEINFO)mp2)->szOldDirName,
                                                          ((PDIRRENAMEINFO)mp2)->szNewDirName );
            break;
          case DTN_CONTEXTMENU:
            if( mp2 ) {
              trace( hwnd, "WM_CONTROL [DTN_CONTEXTMENU] - %s", mp2 );
            } else {
              trace( hwnd, "WM_CONTROL [DTN_CONTEXTMENU]" );
            }
            break;
        }
        return 0;
      }
      break;
  }

  return WinDefDlgProc( hwnd, msg, mp1, mp2 );
}

/* Program entry point.
 */

int main( void )
{
  HAB hab = WinInitialize( 0 );
  HMQ hmq = WinCreateMsgQueue( hab, 0 );

  FOCInitialize();
  WinDlgBox( HWND_DESKTOP, NULLHANDLE, process_dlg, NULLHANDLE, DL_SAMPLE, NULL );

  WinDestroyMsgQueue( hmq );
  WinTerminate( hab );
  return 0;
}

