/*
 * Copyright (C) 2009 eCo Software
 * Copyright (C) 2009 Dmitry A.Steklenev
 */

#define  INCL_WIN
#include <os2.h>
#include <stdlib.h>
#include <stdarg.h>
#include <stdio.h>
#include <string.h>
#include <direct.h>
#include <foc.h>

#include "filedlg.h"

/* Appends specified text to the tracer list box.
 */

static VOID
trace( HWND hwnd, const char* fmt, ... )
{
  CHAR    message[_MAX_PATH+128];
  HWND    tracer = WinWindowFromID( hwnd, LB_TRACER );
  SHORT   pos;
  va_list args;

  va_start( args, fmt );
  vsprintf( message, fmt, args );
  va_end( args );

  if((SHORT)WinSendMsg( tracer, LM_QUERYITEMCOUNT, 0, 0 ) == 32767 ) {
    WinSendMsg( tracer, LM_DELETEITEM, 0, 0 );
  }

  pos = (SHORT)WinSendMsg( tracer, LM_INSERTITEM, MPFROMSHORT( LIT_END ), MPFROMP( message ));
  WinSendMsg( tracer, LM_SELECTITEM,  MPFROMSHORT( pos ), MPFROMLONG( TRUE ));
}

/* Formats dialog presentation window.
 */

static void
format_dlg( HWND hwnd, ULONG msg, MPARAM mp1, MPARAM mp2 )
{
  static HWND  tracer       = NULLHANDLE;
  static SHORT tracer_right = 0;
  static SHORT tracer_top   = 0;

  switch( msg ) {
    case WM_INITDLG:
    {
      SWP swp_dialog;
      SWP swp;

      WinLoadMenu( hwnd, NULLHANDLE, MN_SAMPLE );
      WinSendMsg ( hwnd, WM_UPDATEFRAME, MPFROMLONG( FCF_MENU ), 0 );

      tracer = WinWindowFromID( hwnd, LB_TRACER );

      WinQueryWindowPos( hwnd, &swp_dialog );
      WinQueryWindowPos( tracer, &swp );
      tracer_right = swp_dialog.cx - swp.cx;
      tracer_top   = swp_dialog.cy - swp.cy;
      break;
    }

    case WM_WINDOWPOSCHANGED:
    {
      PSWP pswp_dialog = (PSWP)mp1;
      if( pswp_dialog->fl & SWP_SIZE )
      {
        SWP swp;
        WinQueryWindowPos( tracer, &swp );
        swp.cx = pswp_dialog->cx - tracer_right;
        swp.cy = pswp_dialog->cy - tracer_top;
        WinSetWindowPos( tracer, 0, 0, 0, swp.cx, swp.cy, SWP_SIZE );
      }
      break;
    }
  }
}

/* Sends a list of selected files to the tracer list box.
 */

static VOID
process_files( HWND hwnd, PFOCFILEDLG pfocdlg )
{
  ULONG i;

  if( pfocdlg->lReturn == DID_OK ) {
    trace( hwnd, "FOCFileDlg return szFullFile: %s", pfocdlg->szFullFile );

    if( pfocdlg->papszFQFilename ) {
      for( i = 0; i < pfocdlg->ulFQFCount; i++ ) {
        trace( hwnd, "FOCFileDlg return papszFQFilename[%02d]: %s", i, (*pfocdlg->papszFQFilename)[i] );
      }
    }

    if( pfocdlg->papszFQFilename ) {
      FOCFreeFileDlgList( pfocdlg->papszFQFilename );
    }
  }
}

/* Processes the dialog messages.
 */

static MRESULT EXPENTRY
process_dlg( HWND hwnd, ULONG msg, MPARAM mp1, MPARAM mp2 )
{
  switch( msg ) {
    case WM_INITDLG:
    case WM_WINDOWPOSCHANGED:
      format_dlg( hwnd, msg, mp1, mp2 );
      break;

    case WM_COMMAND:
    {
      HWND hDlg;

      switch( SHORT1FROMMP( mp1 )) {
        case ID_OPEN_FILEDLG:
        {
          FOCFILEDLG focdlg = { sizeof( FOCFILEDLG )};
          PSZ apszTypes[]   = {{ "Audio Stream" },
                               { "Images\t*.JPG; *.PNG;*.GIF" },
                               { "Audios\t*.MP3; *.WAV;*.OGG" },
                               { NULL }};

          focdlg.fl = FDS_OPEN_DIALOG | FDS_CENTER | FDS_MULTIPLESEL;
          focdlg.papszITypeList = (PAPSZ)apszTypes;
          focdlg.pszIType = "Plain Text";
          _getcwd( focdlg.szFullFile, sizeof( focdlg.szFullFile ));
          strcat( focdlg.szFullFile, "\\filedlg.exe" );
          hDlg = FOCFileDlg( HWND_DESKTOP, hwnd, &focdlg );
          trace( hwnd, "FOCFileDlg return %08X, lReturn is %d", hDlg, focdlg.lReturn );
          process_files( hwnd, &focdlg );
          break;
        }

        case ID_SAVE_FILEDLG:
        {
          static FOCFILEDLG focdlg = { sizeof( FOCFILEDLG )};
          focdlg.fl = FDS_SAVEAS_DIALOG | FDS_APPLYBUTTON | FDS_HELPBUTTON;
          strcpy( focdlg.szFullFile, "*.exe" );
          hDlg = FOCFileDlg( HWND_DESKTOP, hwnd, &focdlg );
          trace( hwnd, "FOCFileDlg return %08X, lReturn is %d", hDlg, focdlg.lReturn );
          process_files( hwnd, &focdlg );
          break;
        }

        case ID_SAVE_FILEDLG_CUSTOM:
        {
          FOCFILEDLG focdlg = { sizeof( FOCFILEDLG )};
          focdlg.fl = FDS_SAVEAS_DIALOG | FDS_CENTER | FDS_CUSTOM;
          focdlg.usDlgId = DL_SAMPLE_FILE_DIALOG;
          focdlg.pszTitle = "Customized save as dialog";
          focdlg.pszOKButton = "~Open";
          hDlg = FOCFileDlg( HWND_DESKTOP, hwnd, &focdlg );
          trace( hwnd, "FOCFileDlg return %08X, lReturn is %d", hDlg, focdlg.lReturn );
          process_files( hwnd, &focdlg );
          break;
        }

        case ID_OPEN_SELDIR:
        {
          FOCSELDIR focdlg = { sizeof( FOCSELDIR )};

          focdlg.fl = FDS_CENTER;
          _getcwd( focdlg.szFullDir, sizeof( focdlg.szFullDir ));
          hDlg = FOCSelectDir( HWND_DESKTOP, hwnd, &focdlg );
          trace( hwnd, "FOCSelectDir return %08X, lReturn is %d", hDlg, focdlg.lReturn );
          if( focdlg.lReturn == DID_OK ) {
             trace( hwnd, "FOCSelectDir return szFullDir: %s", focdlg.szFullDir );
          }
          break;
        }
      }
      return 0;
    }
  }

  return WinDefDlgProc( hwnd, msg, mp1, mp2 );
}

/* Program entry point.
 */

int main( void )
{
  HAB hab = WinInitialize( 0 );
  HMQ hmq = WinCreateMsgQueue( hab, 0 );

  FOCInitialize();
  WinDlgBox( HWND_DESKTOP, NULLHANDLE, process_dlg, NULLHANDLE, DL_SAMPLE, NULL );

  WinDestroyMsgQueue( hmq );
  WinTerminate( hab );
  return 0;
}

