/*
 * Copyright (C) 2009 eCo Software
 * Copyright (C) 2009 Dmitry A.Steklenev
 */

#define  INCL_WIN
#include <os2.h>
#include <stdlib.h>
#include <stdarg.h>
#include <stdio.h>
#include <foc.h>

#include "fileview.h"

/* Appends specified text to the tracer list box.
 */

static VOID
trace( HWND hwnd, const char* fmt, ... )
{
  CHAR    message[_MAX_PATH+128];
  HWND    tracer = WinWindowFromID( hwnd, LB_TRACER );
  SHORT   pos;
  va_list args;

  va_start( args, fmt );
  vsprintf( message, fmt, args );
  va_end( args );

  if((SHORT)WinSendMsg( tracer, LM_QUERYITEMCOUNT, 0, 0 ) == 32767 ) {
    WinSendMsg( tracer, LM_DELETEITEM, 0, 0 );
  }

  pos = (SHORT)WinSendMsg( tracer, LM_INSERTITEM, MPFROMSHORT( LIT_END ), MPFROMP( message ));
  WinSendMsg( tracer, LM_SELECTITEM,  MPFROMSHORT( pos ), MPFROMLONG( TRUE ));
}

/* Formats dialog presentation window.
 */

static void
format_dlg( HWND hwnd, ULONG msg, MPARAM mp1, MPARAM mp2 )
{
  static HWND  fileview       = NULLHANDLE;
  static SHORT fileview_right = 0;
  static SHORT fileview_top   = 0;
  static HWND  path           = NULLHANDLE;
  static SHORT path_right     = 0;
  static HWND  tracer         = NULLHANDLE;
  static SHORT tracer_right   = 0;

  switch( msg ) {
    case WM_INITDLG:
    {
      SWP swp_dialog;
      SWP swp;

      WinLoadMenu( hwnd, NULLHANDLE, MN_SAMPLE );
      WinSendMsg ( hwnd, WM_UPDATEFRAME, MPFROMLONG( FCF_MENU ), 0 );

      fileview = WinWindowFromID( hwnd, FV_FILEVIEW );
      path     = WinWindowFromID( hwnd, EF_PATH     );
      tracer   = WinWindowFromID( hwnd, LB_TRACER   );

      WinQueryWindowPos( hwnd, &swp_dialog );
      WinQueryWindowPos( fileview, &swp );
      fileview_right = swp_dialog.cx - swp.cx;
      fileview_top = swp_dialog.cy - swp.cy;
      WinQueryWindowPos( path, &swp );
      path_right = swp_dialog.cx - swp.cx;
      WinQueryWindowPos( tracer, &swp );
      tracer_right = swp_dialog.cx - swp.cx;
      break;
    }

    case WM_WINDOWPOSCHANGED:
    {
      PSWP pswp_dialog = (PSWP)mp1;
      if( pswp_dialog->fl & SWP_SIZE )
      {
        SWP swp;

        WinQueryWindowPos( fileview, &swp );
        swp.cx = pswp_dialog->cx - fileview_right;
        swp.cy = pswp_dialog->cy - fileview_top;
        WinSetWindowPos( fileview, 0, 0, 0, swp.cx, swp.cy, SWP_SIZE );

        WinQueryWindowPos( path, &swp );
        swp.cx = pswp_dialog->cx - path_right - 6;
        WinSetWindowPos( path, 0, 0, 0, swp.cx, swp.cy - 6, SWP_SIZE );

        WinQueryWindowPos( tracer, &swp );
        swp.cx = pswp_dialog->cx - tracer_right;
        WinSetWindowPos( tracer, 0, 0, 0, swp.cx, swp.cy, SWP_SIZE );
      }
      break;
    }
  }
}

/* Processes the dialog messages.
 */

static MRESULT EXPENTRY
process_dlg( HWND hwnd, ULONG msg, MPARAM mp1, MPARAM mp2 )
{
  switch( msg ) {
    case WM_INITDLG:
    case WM_WINDOWPOSCHANGED:
      format_dlg( hwnd, msg, mp1, mp2 );
      break;

    case WM_COMMAND:
      switch( SHORT1FROMMP( mp1 )) {
        case ID_QUERY_CURDIR:
        {
          CHAR szFileName[CCHMAXPATH];
          if( LONGFROMMR( WinSendDlgItemMsg( hwnd, FV_FILEVIEW,
                          FVM_QUERYCURDIR, MPFROMLONG( CCHMAXPATH ), MPFROMP( szFileName ))) > 0 )
          {
            WinSetDlgItemText( hwnd, EF_PATH, szFileName );
          }
          break;
        }

        case ID_QUERY_CURSORED:
        {
          CHAR szFileName[CCHMAXPATH];
          if( LONGFROMMR( WinSendDlgItemMsg( hwnd, FV_FILEVIEW,
                          FVM_QUERYCURSORED, MPFROMLONG( CCHMAXPATH ), MPFROMP( szFileName ))) > 0 )
          {
            WinSetDlgItemText( hwnd, EF_PATH, szFileName );
          }
          break;
        }

        case ID_QUERY_SELECTED:
        {
          FILELIST fList = { sizeof( FILELIST )};
          if( WinSendDlgItemMsg( hwnd, FV_FILEVIEW, FVM_QUERYSELECTED, MPFROMP( &fList ), 0 ))
          {
            ULONG i;
            for( i = 0; i < fList.ulFQFCount; i++ ) {
              trace( hwnd, "SELECTED - %s", (*fList.papszFQFilename)[i] );
            }
            WinSendDlgItemMsg( hwnd, FV_FILEVIEW,
                               FVM_FREEFILELIST, MPFROMP( &fList ), 0 );
          }
          break;
        }

        case ID_REFRESH:
          WinSendDlgItemMsg( hwnd, FV_FILEVIEW, FVM_REFRESH, 0, 0 );
          break;


        case ID_DELETE_SELECTED:
        {
          FILELIST fList = { sizeof( FILELIST )};
          if( WinSendDlgItemMsg( hwnd, FV_FILEVIEW, FVM_QUERYSELECTED, MPFROMP( &fList ), 0 ))
          {
            WinSendDlgItemMsg( hwnd, FV_FILEVIEW, FVM_REQUESTDELETE, MPFROMP( &fList ), 0 );
            WinSendDlgItemMsg( hwnd, FV_FILEVIEW, FVM_FREEFILELIST,  MPFROMP( &fList ), 0 );
          }
          break;
        }

        case ID_RENAME_CURSORED:
        {
          CHAR szFileName[CCHMAXPATH];
          if( LONGFROMMR( WinSendDlgItemMsg( hwnd, FV_FILEVIEW,
                          FVM_QUERYCURSORED, MPFROMLONG( CCHMAXPATH ), MPFROMP( szFileName ))) > 0 )
          {
            WinSendDlgItemMsg( hwnd, FV_FILEVIEW, FVM_REQUESTEDIT, MPFROMP( szFileName ), 0 );
          }
          break;
        }

        case ID_CREATENEWDIR:
          WinSendDlgItemMsg( hwnd, FV_FILEVIEW, FVM_REQUESTNEWDIR, 0, 0 );
          break;

        case ID_VIEW_AS_ICONS:
          WinSendDlgItemMsg( hwnd, FV_FILEVIEW, FVM_SETVIEWSTYLE, MPFROMLONG( FVS_ICON ), 0 );
          break;

        case ID_VIEW_AS_DETAILS:
          WinSendDlgItemMsg( hwnd, FV_FILEVIEW, FVM_SETVIEWSTYLE, MPFROMLONG( FVS_DETAIL ), 0 );
          break;

        case ID_SORT_BY_NONE:
          WinSendDlgItemMsg( hwnd, FV_FILEVIEW, FVM_SORT, MPFROMLONG( FVA_NONE ), 0 );
          break;
        case ID_SORT_BY_NAME:
          WinSendDlgItemMsg( hwnd, FV_FILEVIEW, FVM_SORT, MPFROMLONG( FVA_NAME ), 0 );
          break;
        case ID_SORT_BY_SIZE:
          WinSendDlgItemMsg( hwnd, FV_FILEVIEW, FVM_SORT, MPFROMLONG( FVA_SIZE ), 0 );
          break;
        case ID_SORT_BY_TIME:
          WinSendDlgItemMsg( hwnd, FV_FILEVIEW, FVM_SORT, MPFROMLONG( FVA_TIME ), 0 );
          break;
      }
      return 0;

    case WM_CHAR:
      if(( SHORT1FROMMP(mp1) & KC_VIRTUALKEY ) && !( SHORT1FROMMP(mp1) & ( KC_KEYUP | KC_PREVDOWN ))) {
        if( SHORT2FROMMP(mp2) == VK_ENTER || SHORT2FROMMP(mp2) == VK_NEWLINE )
        {
          CHAR pathname[CCHMAXPATH];
          WinQueryDlgItemText( hwnd, EF_PATH, CCHMAXPATH, pathname );
          WinSendDlgItemMsg( hwnd, FV_FILEVIEW, FVM_CHDIR, MPFROMP( pathname ), 0 );
        }
      }
      break;

    case WM_CONTROL:
      if( SHORT1FROMMP(mp1) == FV_FILEVIEW ) {
        switch( SHORT2FROMMP(mp1)) {
          case FVN_CHDIR:
            trace( hwnd, "WM_CONTROL [FVN_CHDIR   ] - %s", mp2 );
            break;
          case FVN_ENTER:
            trace( hwnd, "WM_CONTROL [FVN_ENTER   ] - %s", mp2 );
            break;
          case FVN_SELECT:
            trace( hwnd, "WM_CONTROL [FVN_SELECT  ] - %s", mp2 );
            break;
          case FVN_DESELECT:
            trace( hwnd, "WM_CONTROL [FVN_DESELECT] - %s", mp2 );
            break;
          case FVN_DELETED:
            trace( hwnd, "WM_CONTROL [FVN_DELETED ] - %s", mp2 );
            break;
          case FVN_NEW:
            trace( hwnd, "WM_CONTROL [FVN_NEW     ] - %s", mp2 );
            break;
          case FVN_RENAMED:
            trace( hwnd, "WM_CONTROL [FVN_RENAMED ] - %s -> %s",
                                                           ((PFILERENAMEINFO)mp2)->szOldFileName,
                                                           ((PFILERENAMEINFO)mp2)->szNewFileName );
            break;
          case FVN_CONTEXTMENU:
            if( mp2 ) {
              trace( hwnd, "WM_CONTROL [FVN_CONTEXTMENU] - %s", mp2 );
            } else {
              trace( hwnd, "WM_CONTROL [FVN_CONTEXTMENU]" );
            }
            break;
        }
        return 0;
      }
      break;
  }

  return WinDefDlgProc( hwnd, msg, mp1, mp2 );
}

/* Program entry point.
 */

int main( void )
{
  HAB hab = WinInitialize( 0 );
  HMQ hmq = WinCreateMsgQueue( hab, 0 );

  FOCInitialize();
  WinDlgBox( HWND_DESKTOP, NULLHANDLE, process_dlg, NULLHANDLE, DL_SAMPLE, NULL );

  WinDestroyMsgQueue( hmq );
  WinTerminate( hab );
  return 0;
}

